@@ -8,6 +8,7 @@ from types import TracebackType
 from typing import List, Optional, Type
 
 import aiohttp
+from aiohttp import ClientSession
 
 from intellifire4py.const import IntellifireCommand, _log
 from intellifire4py.control import ApiCallException, InputRangeException, LoginException
@@ -28,18 +29,30 @@ class IntellifireControlAsync:
         self, fireplace_ip: str, *, use_http: bool = False, verify_ssl: bool = True
     ) -> None:
         """Init the control class."""
-        self._client = aiohttp.ClientSession()
+        self.__client = aiohttp.ClientSession()
         self._cookie = None
         self.send_mode = IntellifireSendMode.LOCAL
         self.is_logged_in = False
         self._ip = fireplace_ip
         self.default_fireplace: IntellifireFireplace
+        self._last_thermostat_setpoint: int = 0
         if use_http:
             self.prefix = "http"
         else:
             self.prefix = "https"
         self._verify_ssl = verify_ssl
 
+        _log.info("Instantiating a IntellifireControlAsync")
+
+    @property
+    def _client(self) -> ClientSession:
+        """Returns either an open session or a new one - using previous session cookies"""
+        if self.__client.closed:
+            _log.info("Recreating closed Client session using stored cookies")
+            self.__client = aiohttp.ClientSession(cookies=self._cookie)
+        return self.__client
+
+
     async def close(self) -> None:
         """Close socket."""
         return await self._client.close()
@@ -62,7 +75,6 @@ class IntellifireControlAsync:
     async def login(self, *, username: str, password: str) -> None:
         """Run login flow to iftapi.net in order to request cookies."""
         data = f"username={username}&password={password}"
-
         try:
             async with self._client.post(
                 f"{self.prefix}://iftapi.net/a//login", data=data.encode(), ssl=False
@@ -72,10 +84,12 @@ class IntellifireControlAsync:
 
                 self._cookie = resp.cookies  # type: ignore
                 self.is_logged_in = True
+                _log.info("Success - Logged into IFTAPI")
 
             # Now set the default fireplace
             await self._set_default_fireplace()
         except LoginException as ex:
+            _log.warning("Login failure")
             raise ex
         return None
 
@@ -102,6 +116,7 @@ class IntellifireControlAsync:
         locations = await self.get_locations()
         fireplaces = await self.get_fireplaces(location_id=locations[0]["location_id"])
         self.default_fireplace = fireplaces[0]
+        _log.debug(f"configure default fireplace: {self.default_fireplace.serial}")
 
     async def get_locations(self) -> List:
         """Enumerate configured locations that a user has access to.
@@ -240,6 +255,54 @@ class IntellifireControlAsync:
             fireplace=fireplace, command=IntellifireCommand.FAN_SPEED, value=0
         )
 
+    async def turn_off_thermostat(self, *, fireplace: IntellifireFireplace) -> None:
+        """Turn off thermostat mode"""
+        await self.send_command(
+            fireplace=fireplace, command=IntellifireCommand.THERMOSTAT_SETPOINT, value=0
+        )
+
+    async def turn_on_thermostat(self, *, fireplace: IntellifireFireplace) -> None:
+        """Turn on thermostat mode"""
+        await self.send_command(
+            fireplace=fireplace,
+            command=IntellifireCommand.THERMOSTAT_SETPOINT,
+            value=self._last_thermostat_setpoint,
+        )
+
+    async def set_thermostat_f(
+        self, *, fireplace: IntellifireFireplace, temp_f: int
+    ) -> None:
+        """Set thermostat value in fahrenheit."""
+        temp_c = int((temp_f - 32) * 5 / 9)
+        await self.set_thermostat_c(fireplace=fireplace, temp_c=temp_c)
+
+    async def set_thermostat_c(
+        self, *, fireplace: IntellifireFireplace, temp_c: int
+    ) -> None:
+        """Set thermostat value in centigrade."""
+        self._last_thermostat_setpoint = temp_c
+        await self.send_command(
+            fireplace=fireplace,
+            command=IntellifireCommand.THERMOSTAT_SETPOINT,
+            value=temp_c,
+        )
+
+    async def set_sleep_timer(
+        self, *, fireplace: IntellifireFireplace, minutes: int
+    ) -> None:
+        """Set the sleep timer in minutes."""
+        await self.send_command(
+            fireplace=fireplace,
+            command=IntellifireCommand.TIME_REMAINING,
+            value=minutes * 60,
+        )
+
+    async def stop_sleep_timer(self, *, fireplace: IntellifireFireplace) -> None:
+        """Stop the sleep timer."""
+        await self.send_command(
+            fireplace=fireplace, command=IntellifireCommand.TIME_REMAINING, value=0
+        )
+
     @property
     def user(self) -> str:
         """Get user cookie."""
@@ -287,6 +350,12 @@ async def main() -> None:
         fireplace: IntellifireFireplace = fireplaces[0]
         default_fireplace = ift_control.default_fireplace
 
+        print("Closing Session")
+        await ift_control.close()
+        fireplaces = await ift_control.get_fireplaces(location_id=location_id)
+        username = await ift_control.get_username()
+        print("username", username)
+
         print("Serial:", default_fireplace.serial)
         print("APIKey:", default_fireplace.apikey)
 
